from upemtk import *
from itertools import *

from morpho import *
from image import *
from time import time

class Interface:

	def __init__(self, img_start, img_end):
		self.img_start = img_start
		self.img_end = img_end
		self.delaunay = Delaunay(img_start, img_end)
		self.mouse_event = dict()
		self.key_event = dict()
		self.has_quit = False
		self.screen_objects = list()
		self.last_clic = None
		self.need_refresh = True
		
		self.frames = 16
		self.draw_img_ref = list()
		
		cree_fenetre(width * 2, height)
		
		for img, i in zip([img_start, img_end], count()): # We print the two images at the first time only
			self.draw_img_ref.append(image(((2 * i + 1) * width) // 2, height // 2, img))
	
	# Note that actions are functions which takes a (Event, Interface) as parameters
	
	def add_mouse_event(self, name, action):
		self.mouse_event[name] = action
	
	def add_key_event(self, name, action):
		self.key_event[name] = action
		
	def draw_obj(self, obj):
		self.screen_objects.append(obj)
	
	def clean_obj(self):
		for e in self.screen_objects:
			efface(e)
		self.screen_objects = list()
	
	def quit(self):
		if self.has_quit : return
		try:
			ferme_fenetre()
		except:
			pass # We just ignore the exception raised in the case where the user clicked on the window cross
		self.has_quit = True
	
	def active(self):
		return not self.has_quit
	
	def set_frames(self, frames):
		self.frames = frames
	
	def set_image(self, name, path):
		if name == "start": self.img_start = path
		elif name == "end" : self.img_end = path
		
		for e in self.draw_img_ref:
			efface(e)
		for img, i in zip([self.img_start, self.img_end], count()):
			self.draw_img_ref.append(image(((2 * i + 1) * width) // 2, height // 2, img))
	
	def add_coord(self, x, y):
		if self.last_clic == None:
			if x >= width: return
			self.last_clic = (x, y)
		else:
			if x < width: return
			self.delaunay.add_coords(self.last_clic, (x - width, y))
			self.last_clic = None
	
	def draw(self):
		if not self.need_refresh:
			return
			
		self.clean_obj()
		
		for P_start, P_end in self.delaunay.get_points():
			for P, offset in zip([P_start, P_end], [0, width]):
				self.draw_obj(cercle(P[0] + offset, P[1], 3, couleur = "red", remplissage = "red"))
		for T_start, T_end in self.delaunay.get_triangles():
			for T, offset in zip([T_start, T_end], [0, width]):
				for i in range(3):
					self.draw_obj(ligne(T[i][0] + offset, T[i][1], T[(i + 1) % 3][0] + offset, T[(i + 1) % 3][1], couleur = "red"))
		
		offset = width if self.last_clic != None else 0
		self.draw_obj(rectangle(offset, 0, offset + width - 1, height - 1, couleur = "green"))
		if self.last_clic != None:
			self.draw_obj(cercle(self.last_clic[0], self.last_clic[1], 3, couleur = "green", remplissage = "green"))
		
		self.need_refresh = False
	
	def build_frame(self, tau, name, start, end):
		print("Process %s" % name)
		chrono = time()
		trv = Image()
		for x in range(width):
			bars = rectangle(0, height - 5, width * 2, height, remplissage =  "#707070"), \
				rectangle(0, height - 5, 2 * x, height, remplissage =  "#007000")
			try:
				mise_a_jour()
			except:
				self.quit()
				return
			for y in range(height):
				Ps, Pe = self.delaunay.get_morpho_start_and_end((x, y), tau)
				pxs = start.getpixel(Ps)
				pxe = end.getpixel(Pe)
				px = tuple(map(int, [(1 - tau) * s + tau * e for s, e in zip(pxs, pxe)]))
				trv.setpixel((x, y), px)
			for b in bars:
				efface(b)
		print("Rendering in %r s" % (time() - chrono))
		trv.save(name + ".png", "PNG")
		self.draw_obj(image(width, height // 2, name + ".png"))
		try:
			mise_a_jour()
		except:
			self.quit()
			return
	
	def build_morpho(self):
		start = Image(self.img_start)
		end = Image(self.img_end)
		print("%d frames to do" % self.frames)
		try:
			for i in range(1, self.frames):
				tau = i / self.frames
				self.build_frame(tau, "media/frame_%03d" % i, start, end)
		except:
			self.quit()
			return
		print("Done")
	
	def event(self):
		ev = donne_evenement()
		
		if type_evenement(ev) in self.mouse_event:
			self.mouse_event[type_evenement(ev)](ev, self)
			self.need_refresh = True
			
		elif type_evenement(ev) == 'Touche' and touche(ev) in self.key_event:
			self.key_event[touche(ev)](ev, self)
			self.need_refresh = True
		
		try:
			mise_a_jour()
		except:
			self.quit()
	
	
	