from itertools import *
from math_stuff import *
from math import *
from image import *

def resolve_triangle(P_list, t):
	return tuple([P_list[i] for i in t])

def resolve_triangles(P_list, T_list):
	return [resolve_triangle(P_list, t) for t in T_list]

def get_angle(V1, V2):
	val = scalar_prod(V1, V2) / (sqrt(scalar_prod(V1, V1) * scalar_prod(V2, V2)))
	return acos(val)

def get_common_diff(T1, T2):
	common = list(set(T1).intersection(set(T2)))
	diff = list(set(T1).symmetric_difference(set(T2)))
	return (common, diff)

def is_neighboor(T1, T2):
	c, d = get_common_diff(T1, T2)
	return len(c) == 2 and len(d) == 2

def check_delaunay(P_list, T1, T2):
	c, d = get_common_diff(T1, T2)
	T = resolve_triangles(P_list, [(e, c[0], c[1]) for e in d])
	return sum([get_angle(sub(e[1], e[0]), sub(e[2], e[0])) for e in T]) < pi

def switch_triangles(T1, T2):
	c, d = get_common_diff(T1, T2)
	return tuple([(k, d[0], d[1]) for k in c])

class Delaunay:
	
	def __init__(self, img_start, img_end):
		self.start = [(0, 0), (width, 0), (width, height), (0, height)]
		self.end = [e for e in self.start]
		self.trv = dict()
		self.img_start = img_start
		self.img_end = img_end
		self.triangles = [(0, 1, 2), (0, 2, 3)]
	
	def add_coords(self, P1, P2):
		split_t = self.get_in_triangle(P1)
		if split_t == None:
			return
			
		self.start.append(P1)
		self.end.append(P2)
		
		split_t = self.triangles.pop(split_t)
		for i in range(3):
			self.triangles.append(split_t[:i] + (len(self.start) - 1,) + split_t[i + 1:])
		
		todo = self.triangles[-3:]
		while len(todo) > 0:
			t = todo.pop()
			for n in self.get_neighboor(t):
				if not check_delaunay(self.start, t, n):
					self.triangles.remove(t)
					self.triangles.remove(n)
					t, n = switch_triangles(t, n)
					for k in [t, n]:
						todo.append(k)
						self.triangles.append(k)
					break
	
	def get_morphing_points(self, tau):
		if tau in self.trv : return self.trv[tau]
		self.trv[tau] = [tuple([(1 - tau) * s + tau * e for s, e in \
			zip(self.start[i], self.end[i])]) for i in range(len(self.start))]
		return self.trv[tau]
	
	def get_morpho_start_and_end(self, P, tau):
		T = self.get_in_triangle(P, self.get_morphing_points(tau))
		if T == None:
			return (P, P)
		T = self.triangles[T]
		bc = barycentric_coordinates(P, resolve_triangle(self.trv[tau], T))
		return (cartesian_coordinates(bc, resolve_triangle(self.start, T)), \
			cartesian_coordinates(bc, resolve_triangle(self.end, T)))
	
	def get_in_triangle(self, P, area = None):
		if area == None : area = self.start
		for t, t_id in zip(resolve_triangles(area, self.triangles), count()):
			if in_triangle(P, t):
				return t_id
		return None
	
	def get_neighboor(self, t):
		return [k for k in self.triangles if is_neighboor(t, k)]
	
	def get_points(self):
		return zip(self.start, self.end)
	
	def get_triangles(self):
		return zip(*[[[P[e] for e in t] for t in self.triangles] for P in [self.start, self.end]])